// privchk.cpp
//
// Privilege check sample.
//
// $Id: $
//

#include "stdafx.h"

//---------------------------------------------------------------------------
// IsPrivilege
//
//  Checks if the privilege is present in the caller's token.
//
//  Parameters:
//	  pszPrivilegeName - name of the privilege
//
//  Returns:
//	  TRUE, if the privilege is present, FALSE - if it isn't, or if an
//	  error has occured.
//
BOOL
IsPrivilege(
	IN PCTSTR pszPrivilegeName
	)
{
	_ASSERTE(pszPrivilegeName != NULL);

	LUID Luid;
	HANDLE hToken;
	DWORD cbNeeded;
	PTOKEN_PRIVILEGES pPriv;

	if (!LookupPrivilegeValue(NULL, pszPrivilegeName, &Luid))
		return FALSE;

	// obtain the token of the current thread 
	if (!OpenThreadToken(GetCurrentThread(), TOKEN_QUERY, FALSE, &hToken))
	{
		if (GetLastError() != ERROR_NO_TOKEN)
			return FALSE;

		// revert to the process token
		if (!OpenProcessToken(GetCurrentProcess(), TOKEN_QUERY, &hToken))
			return FALSE;
	}

	// determine size of the buffer required to receive all privileges
	if (!GetTokenInformation(hToken, TokenPrivileges, NULL, 0, &cbNeeded))
	{
		if (GetLastError() != ERROR_INSUFFICIENT_BUFFER)
		{
			DWORD dwError = GetLastError();
			_VERIFY(CloseHandle(hToken));
			return SetLastError(dwError), FALSE;
		}
	}

	// allocate privileges buffer
	pPriv = (PTOKEN_PRIVILEGES)_alloca(cbNeeded);
	_ASSERTE(pPriv != NULL);

	// retrieve token privileges
	if (!GetTokenInformation(hToken, TokenPrivileges, pPriv, cbNeeded,
							 &cbNeeded))
	{
		DWORD dwError = GetLastError();
		_VERIFY(CloseHandle(hToken));
		return SetLastError(dwError), FALSE;
	}

	_VERIFY(CloseHandle(hToken));

	// walk through all privileges and check if the specified privilege
	// is present in the caller's token
	for (UINT i = 0; i < pPriv->PrivilegeCount; i++)
	{
		if (pPriv->Privileges[i].Luid.LowPart == Luid.LowPart &&
			pPriv->Privileges[i].Luid.HighPart == Luid.HighPart)
			return TRUE;
	}

	SetLastError(ERROR_SUCCESS);
	return FALSE;
}

//---------------------------------------------------------------------------
// IsPrivilegeEnabled
//
//  Checks whether the specified privilege is enabled in the caller's
//  token.
//
//  Parameters:
//	  pszPrivilegeName - name of the privilege
//
//  Returns:
//	  TRUE, if the privilege is present and enabled in the caller's token,
//	  FAKSE - otherwise.
//
BOOL
IsPrivilegeEnabled(
    IN PCTSTR pszPrivilegeName
    )
{
    _ASSERTE(pszPrivilegeName != NULL);

    HANDLE hToken;
    PRIVILEGE_SET PrivSet;
    BOOL bEnabled = FALSE;

    PrivSet.PrivilegeCount = 1;
    PrivSet.Control = 0;
    PrivSet.Privilege[0].Attributes = 0;

    if (!LookupPrivilegeValue(NULL, pszPrivilegeName, 
                              &PrivSet.Privilege[0].Luid))
		return FALSE;

    // obtain the token of the current thread 
    if (!OpenThreadToken(GetCurrentThread(), TOKEN_QUERY, FALSE, &hToken))
    {
	    if (GetLastError() != ERROR_NO_TOKEN)
		    return FALSE;

	    // revert to the process token
	    if (!OpenProcessToken(GetCurrentProcess(), TOKEN_QUERY, &hToken))
		    return FALSE;
    }

    if (!PrivilegeCheck(hToken, &PrivSet, &bEnabled))
    {
        DWORD dwError = GetLastError();
		_VERIFY(CloseHandle(hToken));
		return SetLastError(dwError), FALSE;
    }

    _VERIFY(CloseHandle(hToken));

    SetLastError(0);
    return bEnabled;
}

//---------------------------------------------------------------------------
// PrintError
//
//  Prints an error code and the corresponding description.
//
//  Parameters:
//	  pszMessage - additional message text
//	  dwError    - error code.
//
//  Returns:
//	  one.
//
int
PrintError(
	IN PCTSTR pszMessage,
	IN DWORD dwErrorCode
	)
{
	_tprintf(_T("%s\n"), pszMessage);

	TCHAR szErrorText[512];

	if (FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, NULL, dwErrorCode, 0,
					  szErrorText, countof(szErrorText), NULL))
		_tprintf(_T("Error %d: %s\n"), dwErrorCode, szErrorText);
	else
		_tprintf(_T("Error %d"), dwErrorCode);

	return 1;
}

//---------------------------------------------------------------------------
// ListPrivileges
//
//  Lists all privileges in the current user's token.
//
//  Parameters:
//    none.
//
//  Returns:
//	  no return value.
//
VOID
ListPrivileges()
{
	HANDLE hToken;
	DWORD cbNeeded;
	PTOKEN_PRIVILEGES pPriv;

	// obtain the token of the current process
	if (!OpenProcessToken(GetCurrentProcess(), TOKEN_QUERY, &hToken))
		return;

	// determine size of the buffer required to receive all privileges
	if (!GetTokenInformation(hToken, TokenPrivileges, NULL, 0, &cbNeeded))
	{
		if (GetLastError() != ERROR_INSUFFICIENT_BUFFER)
		{
			_VERIFY(CloseHandle(hToken));
			return;
		}
	}

	// allocate privileges buffer
	pPriv = (PTOKEN_PRIVILEGES)_alloca(cbNeeded);
	_ASSERTE(pPriv != NULL);

	// retrieve token privileges
	if (!GetTokenInformation(hToken, TokenPrivileges, pPriv, cbNeeded,
							 &cbNeeded))
	{
		_VERIFY(CloseHandle(hToken));
		return;
	}

	_VERIFY(CloseHandle(hToken));

	TCHAR szPrivilegeName[80];

	// list all privileges
	for (UINT i = 0; i < pPriv->PrivilegeCount; i++)
	{
		cbNeeded = countof(szPrivilegeName);
		LookupPrivilegeName(NULL, &pPriv->Privileges[i].Luid,
							szPrivilegeName, &cbNeeded);

		_tprintf(_T("%s\n"), szPrivilegeName);
	}
}

//---------------------------------------------------------------------------
// _tmain
//
//  Program entry point.
//
//  Parameters:
//    argc - number of arguments
//	  argv - vector of arguments
//
//  Returns:
//	  program's exit code.
//
int
_tmain(
	int argc,
	_TCHAR * argv[]
	)
{
	if (argc == 1)
		ListPrivileges();
	else
	{
		if (IsPrivilege(argv[1]))
		{
			_tprintf(_T("You have this privilege\n"));

			if (IsPrivilegeEnabled(argv[1]))
				_tprintf(_T("This privilege is enabled\n"));
			else
				_tprintf(_T("This privilege is not enabled\n"));
		}
		else
			_tprintf(_T("You don't have this privilege\n"));
	}

	return 0;
}

#if !defined(_DEBUG) && defined(_UNICODE)

//---------------------------------------------------------------------------
// wprintf
//
//  printf implementation for Release build to avoid inclusion of CRT stuff.
//
//  Parameters:
//	  fmt - format string
//	  ... - optional parameters
//
//  Returns:
//    number of characters printed.
//
int
__cdecl
wprintf(
	const WCHAR * fmt,
	...
	)
{
	va_list va;
	va_start(va, fmt);

	WCHAR szBufferW[1024];
	CHAR szBufferA[1024];

	int ret = wvsprintfW(szBufferW, fmt, va);

	va_end(va);

	ret = WideCharToMultiByte(CP_ACP, 0, szBufferW, ret, szBufferA,
							  countof(szBufferA), NULL, NULL);

	if (!WriteFile(GetStdHandle(STD_OUTPUT_HANDLE), szBufferA, ret,
				   (DWORD *)&ret, NULL))
		return -1;

	return ret;
}

//---------------------------------------------------------------------------
// wmainCRTStartup
//
//  The real process entry point.
//
//  Parameters:
//    none.
//
//  Returns:
//	  this function does not return to the caller.
//
extern "C"
void 
__cdecl 
wmainCRTStartup()
{
	int argc;
	PWSTR * argv = CommandLineToArgvW(GetCommandLineW(), &argc);

	ExitProcess(wmain(argc, argv));
}

#endif
